@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: EphyUtils.cpp,v 1.11 2005/06/02 20:45:15 jframeau Exp $
+ *  $Id: EphyUtils.cpp,v 1.13.4.3 2005/09/21 20:21:30 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -26,6 +26,7 @@
 #include "ephy-embed-shell.h"
 #include "ephy-embed-single.h"
 #include "print-dialog.h"
+#include "ephy-file-helpers.h"
 
 #include <nsIIOService.h>
 #include <nsIURI.h>
@@ -124,38 +125,83 @@ EphyUtils::FindGtkParent (nsIDOMWindow *aDOMWindow)
 	return gtk_widget_get_toplevel (GTK_WIDGET (embed));
 }
 
-#define MM_TO_INCH(x)		(((double) x) / 25.4)
-
 nsresult
-EphyUtils::CollatePrintSettings (const EmbedPrintInfo *info,
+EphyUtils::CollatePrintSettings (EmbedPrintInfo *info,
 				 nsIPrintSettings *options,
 				 gboolean preview)
 {
+	NS_ENSURE_ARG (options);
+
+	/* FIXME: for CUPS printers, print directly instead of to a tmp file? */
+	const static PRUnichar pName[] = { 'P', 'o', 's', 't', 'S', 'c', 'r', 'i', 'p', 't', '/', 'd', 'e', 'f', 'a', 'u', 'l', 't', '\0' };
+	options->SetPrinterName(nsEmbedString(pName).get());
+
 	const static int frame_types[] = {
 		nsIPrintSettings::kFramesAsIs,
 		nsIPrintSettings::kSelectedFrame,
 		nsIPrintSettings::kEachFrameSep
 	};
 
-	switch (info->pages)
+	switch (info->range)
 	{
-		case 0:
+	case GNOME_PRINT_RANGE_CURRENT:
+	case GNOME_PRINT_RANGE_SELECTION_UNSENSITIVE:
+	case GNOME_PRINT_RANGE_ALL:
 		options->SetPrintRange (nsIPrintSettings::kRangeAllPages);
 		break;
-	case 1:
+	case GNOME_PRINT_RANGE_RANGE:
 		options->SetPrintRange (nsIPrintSettings::kRangeSpecifiedPageRange);
 		options->SetStartPageRange (info->from_page);
 		options->SetEndPageRange (info->to_page);
 		break;
-	case 2:
+	case GNOME_PRINT_RANGE_SELECTION:
 		options->SetPrintRange (nsIPrintSettings::kRangeSelection);
 		break;
 	}
+	
+	const GnomePrintUnit *unit, *inch, *mm;
+	double value;
+
+	mm = gnome_print_unit_get_by_abbreviation ((const guchar *) "mm");
+	inch = gnome_print_unit_get_by_abbreviation ((const guchar *) "in");
+	g_assert (mm != NULL && inch != NULL);
+
+	/* top margin */
+	if (gnome_print_config_get_length (info->config,
+					   (const guchar *) GNOME_PRINT_KEY_PAGE_MARGIN_TOP,
+					   &value, &unit)
+	    && gnome_print_convert_distance (&value, unit, inch))
+	{
+		options->SetMarginTop (value);
+	}
+
+	/* bottom margin */
+	if (gnome_print_config_get_length (info->config,
+					   (const guchar *) GNOME_PRINT_KEY_PAGE_MARGIN_BOTTOM,
+					   &value, &unit)
+	    && gnome_print_convert_distance (&value, unit, inch))
+	{
+		options->SetMarginBottom (value);
+	}
+
+	/* left margin */
+	if (gnome_print_config_get_length (info->config,
+					   (const guchar *) GNOME_PRINT_KEY_PAGE_MARGIN_LEFT,
+					   &value, &unit)
+	    && gnome_print_convert_distance (&value, unit, inch))
+	{
+		options->SetMarginLeft (value);
+	}
+
+	/* right margin */
+	if (gnome_print_config_get_length (info->config,
+					   (const guchar *) GNOME_PRINT_KEY_PAGE_MARGIN_RIGHT,
+					   &value, &unit)
+	    && gnome_print_convert_distance (&value, unit, inch))
+	{
+		options->SetMarginRight (value);
+	}
 
-	options->SetMarginTop (MM_TO_INCH (info->top_margin));
-	options->SetMarginBottom (MM_TO_INCH (info->bottom_margin));
-	options->SetMarginLeft (MM_TO_INCH (info->left_margin));
-	options->SetMarginRight (MM_TO_INCH (info->right_margin));
 
 	nsEmbedString tmp;
 
@@ -183,38 +229,114 @@ EphyUtils::CollatePrintSettings (const EmbedPrintInfo *info,
 			   NS_CSTRING_ENCODING_UTF8, tmp);
 	options->SetFooterStrRight(tmp.get());
 
-	NS_CStringToUTF16 (nsEmbedCString(info->file),
-			   NS_CSTRING_ENCODING_UTF8, tmp);
-	options->SetToFileName (tmp.get());
-
-	NS_CStringToUTF16 (nsEmbedCString(info->printer),
-			   NS_CSTRING_ENCODING_UTF8, tmp);
-	options->SetPrinterName (tmp.get());
-
-	/**
-	 * Work around a mozilla bug where paper size & orientation are ignored
-	 * and the specified file is created (containing invalid postscript)
-	 * in print preview mode if we set "print to file" to true.
-	 * See epiphany bug #119818.
-	 */
+	options->SetPrintToFile (PR_FALSE);
+	
 	if (preview)
 	{
 		options->SetPrintToFile (PR_FALSE);
 	}
 	else
 	{
-		options->SetPrintToFile (info->print_to_file);
+		char *cmd, *base;
+		const char *temp_dir;
+		
+		temp_dir = ephy_file_tmp_dir ();
+		base = g_build_filename (temp_dir, "printXXXXXX", NULL);
+		info->tempfile = ephy_file_tmp_filename (base, "ps");
+		g_free (base);
+		
+		if (info->tempfile == NULL) return NS_ERROR_FAILURE;
+		
+		NS_CStringToUTF16 (nsEmbedCString(info->tempfile),
+				   NS_CSTRING_ENCODING_UTF8, tmp);
+		options->SetPrintToFile (PR_TRUE);
+		options->SetToFileName (tmp.get());
 	}
 
-	/* native paper size formats. Our dialog does not support custom yet */
-	options->SetPaperSize (nsIPrintSettings::kPaperSizeNativeData);
+	/* paper size */
+	options->SetPaperSize (nsIPrintSettings::kPaperSizeDefined);
+	options->SetPaperSizeUnit (nsIPrintSettings::kPaperSizeMillimeters);
 
-	NS_CStringToUTF16 (nsEmbedCString(info->paper),
+	/* NOTE: Due to completely braindead mozilla code, the paper
+	 * dimensions aren't actually used. We have to set the paper name,
+	 * and only 6 papers formats are accepted
+	 * (A3, A4, A5, Letter, Legal, Executive).
+	 *
+	 * See http://lxr.mozilla.org/seamonkey/source/gfx/src/ps/nsPostScriptObj.cpp#301 and
+	 * http://lxr.mozilla.org/seamonkey/source/gfx/src/psshared/nsPaperPS.cpp#46 .
+	 */
+	if (gnome_print_config_get_length (info->config,
+					   (const guchar *) GNOME_PRINT_KEY_PAPER_WIDTH,
+					   &value, &unit)
+	   && gnome_print_convert_distance (&value, unit, mm))
+	{
+		options->SetPaperWidth (value);	
+	}
+
+	if (gnome_print_config_get_length (info->config,
+					   (const guchar *) GNOME_PRINT_KEY_PAPER_HEIGHT,
+					   &value, &unit)
+	   && gnome_print_convert_distance (&value, unit, mm))
+	{
+		options->SetPaperHeight (value);	
+	}
+
+	/* Mozilla bug https://bugzilla.mozilla.org/show_bug.cgi?id=307404
+	 * means that we cannot actually use any paper sizes except mozilla's
+	 * builtin list, and we must refer to them *by name*!
+	 */
+#ifndef HAVE_GECKO_1_9
+	/* Gnome-Print names some papers differently than what moz understands */
+	static const struct
+	{
+		const char *gppaper;
+		const char *mozpaper;
+	}
+	paper_table [] =
+	{
+		{ "USLetter", "Letter" },
+		{ "USLegal", "Legal" }
+	};
+#endif /* !HAVE_GECKO_1_9 */
+
+	/* paper name */
+	char *string = (char *) gnome_print_config_get (info->config,
+						  (const guchar *) GNOME_PRINT_KEY_PAPER_SIZE);
+	const char *paper = string;
+
+#ifndef HAVE_GECKO_1_9
+	for (PRUint32 i = 0; i < G_N_ELEMENTS (paper_table); i++)
+	{
+		if (string != NULL &&
+		    g_ascii_strcasecmp (paper_table[i].gppaper, string) == 0)
+		{
+			paper = paper_table[i].mozpaper;
+			break;
+		}
+	}
+#endif /* !HAVE_GECKO_1_9 */
+
+	NS_CStringToUTF16 (nsEmbedCString(paper),
 			   NS_CSTRING_ENCODING_UTF8, tmp);
 	options->SetPaperName (tmp.get());
+	g_free (string);
+
+	/* paper orientation */
+	string = (char *) gnome_print_config_get (info->config,
+						  (const guchar *) GNOME_PRINT_KEY_ORIENTATION);
+	if (string == NULL) string = g_strdup ("R0");
+
+	if (strncmp (string, "R90", 3) == 0 || strncmp (string, "R270", 4) == 0)
+	{
+		options->SetOrientation (nsIPrintSettings::kLandscapeOrientation);
+	}
+	else
+	{
+		options->SetOrientation (nsIPrintSettings::kPortraitOrientation);
+	}
+	g_free (string);
 
 	options->SetPrintInColor (info->print_color);
-	options->SetOrientation (info->orientation);
 	options->SetPrintFrameType (frame_types[info->frame_type]);
 
 	return NS_OK;
