@@ -16,7 +16,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: GlobalHistory.cpp,v 1.36 2005/04/26 12:01:39 chpe Exp $
+ *  $Id: GlobalHistory.cpp,v 1.38 2005/08/25 13:51:44 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -58,11 +58,6 @@ NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aT
 
 	NS_ENSURE_ARG (aURI);
 
-	if (aRedirect || !aToplevel)
-	{
-		return NS_OK;
-	}
-
 	// filter out unwanted URIs such as chrome: etc
 	// The model is really if we don't know differently then add which basically
 	// means we are suppose to try all the things we know not to allow in and
@@ -70,9 +65,7 @@ NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aT
 	// against the most common case we know to allow in and go on and say yes
 	// to it.
 
-	PRBool isHTTP = PR_FALSE;
-	PRBool isHTTPS = PR_FALSE;
-
+	PRBool isHTTP = PR_FALSE, isHTTPS = PR_FALSE;
 	rv = aURI->SchemeIs("http", &isHTTP);
 	rv |= aURI->SchemeIs("https", &isHTTPS);
 	NS_ENSURE_SUCCESS (rv, NS_ERROR_FAILURE);
@@ -96,7 +89,7 @@ NS_IMETHODIMP MozGlobalHistory::AddURI(nsIURI *aURI, PRBool aRedirect, PRBool aT
 
 	nsEmbedCString spec;
 	rv = aURI->GetSpec(spec);
-	NS_ENSURE_SUCCESS(rv, rv);
+	NS_ENSURE_TRUE (NS_SUCCEEDED(rv) && spec.Length(), rv);
 
 	ephy_history_add_page (mGlobalHistory, spec.get());
 	
@@ -132,3 +125,21 @@ NS_IMETHODIMP MozGlobalHistory::SetPageTitle(nsIURI *aURI, const nsAString & aTi
 	
 	return NS_OK;
 }
+
+#ifdef HAVE_GECKO_1_9
+/* unsigned long getURIGeckoFlags(in nsIURI aURI); */
+NS_IMETHODIMP
+GlobalHistory::GetURIGeckoFlags(nsIURI *aURI, PRUint32* aFlags)
+{
+#error Implement me!
+	return NS_ERROR_NOT_IMPLEMENTED:
+}
+
+/* void setURIGeckoFlags(in nsIURI aURI, in unsigned long aFlags); */
+NS_IMETHODIMP
+GlobalHistory::SetURIGeckoFlags(nsIURI *aURI, PRUint32 aFlags)
+{
+#error Implement me!
+	return NS_ERROR_NOT_IMPLEMENTED:
+}
+#endif /* HAVE_GECKO_1_9 */
