@@ -38,7 +38,7 @@
  *
  *  Copyright (C) 2005 Christian Persch
  *
- *  $Id: GtkNSSSecurityWarningDialogs.cpp,v 1.6 2005/01/10 21:05:04 chpe Exp $
+ *  $Id: GtkNSSSecurityWarningDialogs.cpp,v 1.7 2005/07/24 12:19:42 chpe Exp $
  */
 
 #include "mozilla-config.h"
@@ -250,7 +250,7 @@ GtkNSSSecurityWarningDialogs::DoDialog (nsIInterfaceRequestor *aContext,
 
 	if (aSecondary)
 	{
-		gtk_message_dialog_format_secondary_markup
+		gtk_message_dialog_format_secondary_text
 			(GTK_MESSAGE_DIALOG (dialog), aSecondary);
 	}
 
