@@ -15,7 +15,7 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-node-view.c,v 1.58 2005/01/21 19:33:02 jbaayen Exp $
+ *  $Id: ephy-node-view.c,v 1.62.4.1 2005/10/01 07:38:27 chpe Exp $
  */
 
 #include "config.h"
@@ -69,8 +69,6 @@ struct _EphyNodeViewPrivate
 	gboolean remove_if_cancelled;
 	int editable_property;
 
-	int searchable_data_column;
-
 	gboolean drag_started;
 	int drag_button;
 	int drag_x;
@@ -311,7 +309,8 @@ drag_motion_cb (GtkWidget *widget,
 				(node, view->priv->priority_prop_id);
 
 		if (priority != EPHY_NODE_VIEW_ALL_PRIORITY &&
-		    priority != EPHY_NODE_VIEW_SPECIAL_PRIORITY)
+		    priority != EPHY_NODE_VIEW_SPECIAL_PRIORITY &&
+		    ephy_node_get_is_drag_source (node))
 		{
 			action = context->suggested_action;
 		}
@@ -376,6 +375,10 @@ drag_data_received_cb (GtkWidget *widget,
 		return;
 	}	
 
+	/* appease GtkTreeView by preventing its drag_data_receive
+	* from being called */
+	g_signal_stop_emission_by_name (view, "drag_data_received");
+
 	if (!view->priv->have_drag_data)
 	{
 		view->priv->have_drag_data = TRUE;
@@ -390,16 +393,18 @@ drag_data_received_cb (GtkWidget *widget,
 		gboolean success = FALSE;
 		GtkTreePath *path;
 
-		gtk_tree_view_get_dest_row_at_pos
-			(GTK_TREE_VIEW (widget), x, y, &path, &pos);
-
-		g_return_if_fail (path != NULL);
+		if (gtk_tree_view_get_dest_row_at_pos
+			(GTK_TREE_VIEW (widget), x, y, &path, &pos) == FALSE)
+		{
+			return;
+		}
 
 		node = get_node_from_path (view, path);
+		if (node == NULL) return;
 
 		uris = gtk_selection_data_get_uris (selection_data);
 
-		if (uris != NULL)
+		if (uris != NULL && ephy_node_get_is_drag_dest (node))
 		{
 			/* FIXME fill success */
 			g_signal_emit (G_OBJECT (view),
@@ -418,12 +423,6 @@ drag_data_received_cb (GtkWidget *widget,
 			gtk_tree_path_free (path);
 		}
 	}
-
-
-	/* appease GtkTreeView by preventing its drag_data_receive
-	 * from being called */
-	g_signal_stop_emission_by_name (GTK_TREE_VIEW (view),
-					"drag_data_received");
 }
 
 static gboolean
@@ -445,7 +444,7 @@ drag_drop_cb (GtkWidget *widget,
 
 void
 ephy_node_view_enable_drag_dest (EphyNodeView *view,
-				 GtkTargetEntry *types,
+				 const GtkTargetEntry *types,
 				 int n_types)
 {
 	GtkWidget *treeview;
@@ -567,13 +566,12 @@ ephy_node_view_key_press_cb (GtkTreeView *treeview,
 			     GdkEventKey *event,
 			     EphyNodeView *view)
 {
-	guint32 unicode;
-
-	unicode = gdk_keyval_to_unicode (event->keyval);
+	gboolean handled = FALSE;
 
 	if (event->keyval == GDK_space ||
 	    event->keyval == GDK_Return ||
-	    event->keyval == GDK_KP_Enter)
+	    event->keyval == GDK_KP_Enter ||
+	    event->keyval == GDK_ISO_Enter)
 	{
 		if (view->priv->toggle_column >= 0)
 		{
@@ -582,15 +580,11 @@ ephy_node_view_key_press_cb (GtkTreeView *treeview,
 			selection = gtk_tree_view_get_selection (treeview);
 			gtk_tree_selection_selected_foreach
 					(selection, path_toggled, view);
+			handled = TRUE;
 		}
 	}
-	else if (view->priv->searchable_data_column != -1 && unicode)
-	{
-		return ephy_gui_select_row_by_key
-			(treeview, view->priv->searchable_data_column, unicode);
-	}
 
-	return FALSE;
+	return handled;
 }
 
 static void
@@ -670,6 +664,42 @@ did_not_drag (EphyNodeView *view,
         }                                                                                                              
 }
 
+typedef struct
+{
+	EphyNodeView *view;
+	gboolean result;
+}
+ForeachData;
+
+static void
+check_node_is_drag_source (GtkTreeModel *model,
+			   GtkTreePath *path,
+			   GtkTreeIter *iter,
+			   ForeachData *data)
+{
+	EphyNode *node;
+
+	node = get_node_from_path (data->view, path);
+	data->result = data->result &&
+		       node != NULL &&
+		       ephy_node_get_is_drag_source (node);
+}
+
+static gboolean
+can_drag_selection (EphyNodeView *view)
+{
+	GtkTreeView *tree_view = GTK_TREE_VIEW (view);
+	GtkTreeSelection *selection;
+	ForeachData data = { view, TRUE };
+
+	selection = gtk_tree_view_get_selection (tree_view);
+	gtk_tree_selection_selected_foreach (selection,
+					     (GtkTreeSelectionForeachFunc) check_node_is_drag_source,
+					     &data);
+
+	return data.result;
+}
+
 static void
 drag_data_get_cb (GtkWidget *widget,
                   GdkDragContext *context,
@@ -714,6 +744,7 @@ button_release_cb (GtkWidget *widget,
                         did_not_drag (view, event);
                         return TRUE;
                 }
+		view->priv->drag_started = FALSE;
         }
         return FALSE;
 }
@@ -734,7 +765,8 @@ motion_notify_cb (GtkWidget *widget,
 	{
 		if (gtk_drag_check_threshold (widget, view->priv->drag_x,
                                               view->priv->drag_y, event->x,
-					      event->y))
+					      event->y)
+		    && can_drag_selection (view))
 		{
 			context = gtk_drag_begin
 				(widget, view->priv->source_target_list,
@@ -1205,12 +1237,6 @@ ephy_node_view_add_column (EphyNodeView *view,
 				  G_CALLBACK (renderer_editing_canceled_cb), view);
 	}
 
-	if ((flags & EPHY_NODE_VIEW_SEARCHABLE) &&
-	    (view->priv->searchable_data_column == -1))
-	{
-		view->priv->searchable_data_column = column;
-	}
-
 	gtk_tree_view_column_pack_start (gcolumn, renderer, TRUE);
 	gtk_tree_view_column_set_attributes (gcolumn, renderer,
 					     "text", column,
@@ -1237,6 +1263,12 @@ ephy_node_view_add_column (EphyNodeView *view,
 		gtk_tree_view_column_set_sort_column_id (gcolumn, column);
 	}
 
+	if (flags & EPHY_NODE_VIEW_SEARCHABLE)
+	{
+		gtk_tree_view_set_search_column (GTK_TREE_VIEW (view), column);
+		gtk_tree_view_set_enable_search (GTK_TREE_VIEW (view), TRUE);
+	}
+
 	if (ret != NULL)
 		*ret = gcolumn;
 
@@ -1283,7 +1315,6 @@ ephy_node_view_init (EphyNodeView *view)
 	view->priv = EPHY_NODE_VIEW_GET_PRIVATE (view);
 
 	view->priv->toggle_column = -1;
-	view->priv->searchable_data_column = -1;
 	view->priv->priority_column = -1;
 	view->priv->priority_prop_id = 0;
 	view->priv->sort_column = -1;
@@ -1411,7 +1442,7 @@ ephy_node_view_select_node (EphyNodeView *view,
 
 void
 ephy_node_view_enable_drag_source (EphyNodeView *view,
-				   GtkTargetEntry *types,
+				   const GtkTargetEntry *types,
 				   int n_types,
 				   int base_drag_column,
 				   int extra_drag_column)
